@@ -1,35 +1,9 @@
 from __future__ import division
-"""frdata.py
-
+"""
 Frequency response data representation and functions.
 
-This file contains the FRD class and also functions that operate on
+This module contains the FRD class and also functions that operate on
 FRD data.
-
-Routines in this module:
-
-FRD.__init__
-FRD.copy
-FRD.__str__
-FRD.__neg__
-FRD.__add__
-FRD.__radd__
-FRD.__sub__
-FRD.__rsub__
-FRD.__mul__
-FRD.__rmul__
-FRD.__div__
-FRD.__rdiv__
-FRD.__truediv__
-FRD.__rtruediv__
-FRD.evalfr
-FRD.freqresp
-FRD.pole
-FRD.zero
-FRD.feedback
-FRD._common_den
-_convertToFRD
-
 """
 
 """Copyright (c) 2010 by California Institute of Technology
@@ -75,23 +49,25 @@ $Id: frd.py 185 2012-08-30 05:44:32Z murrayrm $
 """
 
 # External function declarations
+import numpy as np
 from numpy import angle, array, empty, ones, \
     real, imag, matrix, absolute, eye, linalg, where, dot
 from scipy.interpolate import splprep, splev
-from .lti import Lti
+from .lti import LTI
 
-class FRD(Lti):
-    """The FRD class represents (measured?) frequency response
-    TF instances and functions.
+__all__ = ['FRD', 'frd']
 
-    The FRD class is derived from the Lti parent class.  It is used
-    throughout the python-control library to represent systems in frequency
-    response data form.
+class FRD(LTI):
+    """FRD(d, w)
 
-    The main data members are 'omega' and 'fresp'. omega is a 1D
-    array with the frequency points of the response. fresp is a 3D array,
-    with the first dimension corresponding to the outputs of the FRD,
-    the second dimension corresponding to the inputs, and the 3rd dimension
+    A class for models defined by frequency response data (FRD)
+
+    The FRD class is used to represent systems in frequency response data form.
+
+    The main data members are 'omega' and 'fresp', where `omega` is a 1D
+    array with the frequency points of the response, and `fresp` is a 3D array,
+    with the first dimension corresponding to the output index of the FRD,
+    the second dimension corresponding to the input index, and the 3rd dimension
     corresponding to the frequency points in omega.
     For example,
 
@@ -107,7 +83,9 @@ class FRD(Lti):
     epsw = 1e-8
 
     def __init__(self, *args, **kwargs):
-        """Construct a transfer function.
+        """FRD(d, w)
+
+        Construct an FRD object
 
         The default constructor is FRD(d, w), where w is an iterable of
         frequency points, and d is the matching frequency data.
@@ -118,16 +96,14 @@ class FRD(Lti):
         To call the copy constructor, call FRD(sys), where sys is a
         FRD object.
 
-        To construct frequency response data for an existing Lti
+        To construct frequency response data for an existing LTI
         object, other than an FRD, call FRD(sys, omega)
 
-
-
         """
         smooth = kwargs.get('smooth', False)
 
         if len(args) == 2:
-            if not isinstance(args[0], FRD) and isinstance(args[0], Lti):
+            if not isinstance(args[0], FRD) and isinstance(args[0], LTI):
                 # not an FRD, but still a system, second argument should be
                 # the frequency range
                 otherlti = args[0]
@@ -140,7 +116,7 @@ class FRD(Lti):
                     (otherlti.outputs, otherlti.inputs, numfreq),
                     dtype=complex)
                 for k, w in enumerate(self.omega):
-                    self.fresp[:, :, k] = otherlti.evalfr(w)
+                    self.fresp[:, :, k] = otherlti._evalfr(w)
 
             else:
                 # The user provided a response and a freq vector
@@ -179,7 +155,7 @@ class FRD(Lti):
                         w=1.0/(absolute(self.fresp[i, j, :])+0.001), s=0.0)
         else:
             self.ifunc = None
-        Lti.__init__(self, self.fresp.shape[1], self.fresp.shape[0])
+        LTI.__init__(self, self.fresp.shape[1], self.fresp.shape[0])
 
     def __str__(self):
         """String representation of the transfer function."""
@@ -248,8 +224,9 @@ second has %i." % (self.outputs, other.outputs))
         """Multiply two LTI objects (serial connection)."""
 
         # Convert the second argument to a transfer function.
-        if isinstance(other, (int, float, complex)):
-            return FRD(self.fresp * other, self.omega)
+        if isinstance(other, (int, float, complex, np.number)):
+            return FRD(self.fresp * other, self.omega,
+                       smooth=(self.ifunc is not None))
         else:
             other = _convertToFRD(other, omega=self.omega)
 
@@ -265,14 +242,17 @@ second has %i." % (self.outputs, other.outputs))
                       dtype=self.fresp.dtype)
         for i in range(len(self.omega)):
             fresp[:,:,i] = dot(self.fresp[:,:,i], other.fresp[:,:,i])
-        return FRD(fresp, self.omega)
+        return FRD(fresp, self.omega,
+                   smooth=(self.ifunc is not None) and
+                          (other.ifunc is not None))
 
     def __rmul__(self, other):
         """Right Multiply two LTI objects (serial connection)."""
 
         # Convert the second argument to an frd function.
-        if isinstance(other, (int, float, complex)):
-            return FRD(self.fresp * other, self.omega)
+        if isinstance(other, (int, float, complex, np.number)):
+            return FRD(self.fresp * other, self.omega,
+                       smooth=(self.ifunc is not None))
         else:
             other = _convertToFRD(other, omega=self.omega)
 
@@ -289,14 +269,17 @@ second has %i." % (self.outputs, other.outputs))
                       dtype=self.fresp.dtype)
         for i in range(len(self.omega)):
             fresp[:,:,i] = dot(other.fresp[:,:,i], self.fresp[:,:,i])
-        return FRD(fresp, self.omega)
+        return FRD(fresp, self.omega,
+                   smooth=(self.ifunc is not None) and
+                          (other.ifunc is not None))
 
     # TODO: Division of MIMO transfer function objects is not written yet.
     def __truediv__(self, other):
         """Divide two LTI objects."""
 
-        if isinstance(other, (int, float, complex)):
-            return FRD(self.fresp * (1/other), self.omega)
+        if isinstance(other, (int, float, complex, np.number)):
+            return FRD(self.fresp * (1/other), self.omega,
+                       smooth=(self.ifunc is not None))
         else:
             other = _convertToFRD(other, omega=self.omega)
 
@@ -306,7 +289,9 @@ second has %i." % (self.outputs, other.outputs))
             raise NotImplementedError(
                 "FRD.__truediv__ is currently implemented only for SISO systems.")
 
-        return FRD(self.fresp/other.fresp, self.omega)
+        return FRD(self.fresp/other.fresp, self.omega,
+                   smooth=(self.ifunc is not None) and
+                          (other.ifunc is not None))
 
     # TODO: Remove when transition to python3 complete
     def __div__(self, other):
@@ -315,8 +300,9 @@ second has %i." % (self.outputs, other.outputs))
     # TODO: Division of MIMO transfer function objects is not written yet.
     def __rtruediv__(self, other):
         """Right divide two LTI objects."""
-        if isinstance(other, (int, float, complex)):
-            return FRD(other / self.fresp, self.omega)
+        if isinstance(other, (int, float, complex, np.number)):
+            return FRD(other / self.fresp, self.omega,
+                       smooth=(self.ifunc is not None))
         else:
             other = _convertToFRD(other, omega=self.omega)
 
@@ -335,18 +321,18 @@ second has %i." % (self.outputs, other.outputs))
         if not type(other) == int:
             raise ValueError("Exponent must be an integer")
         if other == 0:
-            return FRD(ones(self.fresp.shape),self.omega) #unity
+            return FRD(ones(self.fresp.shape),self.omega,
+                       smooth=(self.ifunc is not None)) #unity
         if other > 0:
             return self * (self**(other-1))
         if other < 0:
             return (FRD(ones(self.fresp.shape), self.omega) / self) * \
                 (self**(other+1))
 
-
     def evalfr(self, omega):
         """Evaluate a transfer function at a single angular frequency.
 
-        self.evalfr(omega) returns the value of the frequency response
+        self._evalfr(omega) returns the value of the frequency response
         at frequency omega.
 
         Note that a "normal" FRD only returns values for which there is an
@@ -354,7 +340,31 @@ second has %i." % (self.outputs, other.outputs))
         intermediate values.
 
         """
+        warn("FRD.evalfr(omega) will be deprecated in a future release of python-control; use sys.eval(omega) instead", 
+             PendingDeprecationWarning)
+        return self._evalfr(omega)
 
+    # Define the `eval` function to evaluate an FRD at a given (real)
+    # frequency.  Note that we choose to use `eval` instead of `evalfr` to
+    # avoid confusion with :func:`evalfr`, which takes a complex number as its
+    # argument.  Similarly, we don't use `__call__` to avoid confusion between
+    # G(s) for a transfer function and G(omega) for an FRD object.
+    def eval(self, omega):
+        """Evaluate a transfer function at a single angular frequency.
+
+        self._evalfr(omega) returns the value of the frequency response
+        at frequency omega.
+
+        Note that a "normal" FRD only returns values for which there is an
+        entry in the omega vector. An interpolating FRD can return
+        intermediate values.
+
+        """
+        return self._evalfr(omega)
+
+    # Internal function to evaluate the frequency responses
+    def _evalfr(self, omega):
+        """Evaluate a transfer function at a single angular frequency."""
         # Preallocate the output.
         if getattr(omega, '__iter__', False):
             out = empty((self.outputs, self.inputs, len(omega)), dtype=complex)
@@ -367,7 +377,7 @@ second has %i." % (self.outputs, other.outputs))
             except:
                 raise ValueError(
                     "Frequency %f not in frequency list, try an interpolating"
-                    " FRD if you want additional points")
+                    " FRD if you want additional points" % omega)
         else:
             if getattr(omega, '__iter__', False):
                 for i in range(self.outputs):
@@ -403,7 +413,7 @@ second has %i." % (self.outputs, other.outputs))
         omega.sort()
 
         for k, w in enumerate(omega):
-            fresp = self.evalfr(w)
+            fresp = self._evalfr(w)
             mag[:, :, k] = abs(fresp)
             phase[:, :, k] = angle(fresp)
 
@@ -430,14 +440,14 @@ second has %i." % (self.outputs, other.outputs))
                 self.fresp[:, :, k].view(type=matrix),
                 eye(self.inputs))
 
-        return FRD(fresp, other.omega)
+        return FRD(fresp, other.omega, smooth=(self.ifunc is not None))
 
 def _convertToFRD(sys, omega, inputs=1, outputs=1):
     """Convert a system to frequency response data form (if needed).
 
     If sys is already an frd, and its frequency range matches or
     overlaps the range given in omega then it is returned.  If sys is
-    another Lti object or a transfer function, then it is converted to
+    another LTI object or a transfer function, then it is converted to
     a frequency response data at the specified omega. If sys is a
     scalar, then the number of inputs and outputs can be specified
     manually, as in:
@@ -459,17 +469,17 @@ def _convertToFRD(sys, omega, inputs=1, outputs=1):
         raise NotImplementedError(
             "Frequency ranges of FRD do not match, conversion not implemented")
 
-    elif isinstance(sys, Lti):
+    elif isinstance(sys, LTI):
         omega.sort()
         fresp = empty((sys.outputs, sys.inputs, len(omega)), dtype=complex)
         for k, w in enumerate(omega):
-            fresp[:, :, k] = sys.evalfr(w)
+            fresp[:, :, k] = sys._evalfr(w)
 
-        return FRD(fresp, omega)
+        return FRD(fresp, omega, smooth=True)
 
-    elif isinstance(sys, (int, float, complex)):
+    elif isinstance(sys, (int, float, complex, np.number)):
         fresp = ones((outputs, inputs, len(omega)), dtype=float)*sys
-        return FRD(fresp, omega)
+        return FRD(fresp, omega, smooth=True)
 
     # try converting constant matrices
     try:
@@ -479,9 +489,46 @@ def _convertToFRD(sys, omega, inputs=1, outputs=1):
         for i in range(outputs):
             for j in range(inputs):
                 fresp[i,j,:] = sys[i,j]
-        return FRD(fresp, omega)
+        return FRD(fresp, omega, smooth=True)
     except:
         pass
 
     raise TypeError('''Can't convert given type "%s" to FRD system.''' %
                     sys.__class__)
+
+def frd(*args):
+    """frd(d, w)
+
+    Construct a frequency response data model
+
+    frd models store the (measured) frequency response of a system.
+
+    This function can be called in different ways:
+
+    ``frd(response, freqs)``
+        Create an frd model with the given response data, in the form of
+        complex response vector, at matching frequency freqs [in rad/s]
+
+    ``frd(sys, freqs)``
+        Convert an LTI system into an frd model with data at frequencies
+        freqs.
+
+    Parameters
+    ----------
+    response: array_like, or list
+        complex vector with the system response
+    freq: array_lik or lis
+        vector with frequencies
+    sys: LTI (StateSpace or TransferFunction)
+        A linear system
+
+    Returns
+    -------
+    sys: FRD
+        New frequency response system
+
+    See Also
+    --------
+    FRD, ss, tf
+    """
+    return FRD(*args)
