@@ -44,10 +44,12 @@
 import matplotlib.pyplot as plt
 import scipy as sp
 import numpy as np
-from warnings import warn
+import math
 from .ctrlutil import unwrap
 from .bdalg import feedback
-from .lti import isdtime, timebaseEqual
+
+__all__ = ['bode_plot', 'nyquist_plot', 'gangof4_plot',
+           'bode', 'nyquist', 'gangof4']
 
 #
 # Main plotting functions
@@ -58,8 +60,9 @@ from .lti import isdtime, timebaseEqual
 
 # Bode plot
 def bode_plot(syslist, omega=None, dB=None, Hz=None, deg=None,
-        Plot=True, *args, **kwargs):
-    """Bode plot for a system
+        Plot=True, omega_limits=None, omega_num=None, *args, **kwargs):
+    """
+    Bode plot for a system
 
     Plots a Bode plot for the system over a (optional) frequency range.
 
@@ -68,16 +71,21 @@ def bode_plot(syslist, omega=None, dB=None, Hz=None, deg=None,
     syslist : linsys
         List of linear input/output systems (single system is OK)
     omega : freq_range
-        Range of frequencies (list or bounds) in rad/sec
+        Range of frequencies in rad/sec
     dB : boolean
         If True, plot result in dB
     Hz : boolean
         If True, plot frequency in Hz (omega must be provided in rad/sec)
     deg : boolean
-        If True, return phase in degrees (else radians)
+        If True, plot phase in degrees (else radians)
     Plot : boolean
         If True, plot magnitude and phase
-    *args, **kwargs:
+    omega_limits: tuple, list, ... of two values
+        Limits of the to generate frequency vector.
+        If Hz=True the limits are in Hz otherwise in rad/s.
+    omega_num: int
+        number of samples
+    \*args, \**kwargs:
         Additional options to matplotlib (color, linestyle, etc)
 
     Returns
@@ -85,9 +93,9 @@ def bode_plot(syslist, omega=None, dB=None, Hz=None, deg=None,
     mag : array (list if len(syslist) > 1)
         magnitude
     phase : array (list if len(syslist) > 1)
-        phase
+        phase in radians
     omega : array (list if len(syslist) > 1)
-        frequency
+        frequency in rad/sec
 
     Notes
     -----
@@ -115,60 +123,128 @@ def bode_plot(syslist, omega=None, dB=None, Hz=None, deg=None,
     if (not getattr(syslist, '__iter__', False)):
         syslist = (syslist,)
 
-    mags, phases, omegas = [], [], []
+    if omega is None:
+        if omega_limits is None:
+            # Select a default range if none is provided
+            omega = default_frequency_range(syslist, Hz=Hz, number_of_samples=omega_num)
+        else:
+            omega_limits = np.array(omega_limits)
+            if Hz:
+                omega_limits *= 2.*math.pi
+            if omega_num:
+                omega = sp.logspace(np.log10(omega_limits[0]), np.log10(omega_limits[1]), num=omega_num, endpoint=True)
+            else:
+                omega = sp.logspace(np.log10(omega_limits[0]), np.log10(omega_limits[1]), endpoint=True)
+
+    mags, phases, omegas, nyquistfrqs = [], [], [], []
     for sys in syslist:
         if (sys.inputs > 1 or sys.outputs > 1):
-            #TODO: Add MIMO bode plots.
+            # TODO: Add MIMO bode plots.
             raise NotImplementedError("Bode is currently only implemented for SISO systems.")
         else:
-            if omega is None:
-                # Select a default range if none is provided
-                omega = default_frequency_range(syslist)
-
+            omega_sys = np.array(omega)
+            if sys.isdtime(True):
+                nyquistfrq = 2. * math.pi * 1. / sys.dt / 2.
+                omega_sys = omega_sys[omega_sys < nyquistfrq]
+                # TODO: What distance to the Nyquist frequency is appropriate?
+            else:
+                nyquistfrq = None
             # Get the magnitude and phase of the system
-            mag_tmp, phase_tmp, omega_sys = sys.freqresp(omega)
+            mag_tmp, phase_tmp, omega_sys = sys.freqresp(omega_sys)
             mag = np.atleast_1d(np.squeeze(mag_tmp))
             phase = np.atleast_1d(np.squeeze(phase_tmp))
             phase = unwrap(phase)
+            nyquistfrq_plot = None
             if Hz:
-                omega_plot = omega_sys/(2*sp.pi)
+                omega_plot = omega_sys / (2. * math.pi)
+                if nyquistfrq:
+                    nyquistfrq_plot = nyquistfrq / (2. * math.pi)
             else:
                 omega_plot = omega_sys
-            if dB: mag = 20*sp.log10(mag)
-            if deg: phase = phase * 180 / sp.pi
+                if nyquistfrq:
+                    nyquistfrq_plot = nyquistfrq
 
             mags.append(mag)
             phases.append(phase)
             omegas.append(omega_sys)
+            nyquistfrqs.append(nyquistfrq)
             # Get the dimensions of the current axis, which we will divide up
             #! TODO: Not current implemented; just use subplot for now
 
             if (Plot):
+                # Set up the axes with labels so that multiple calls to
+                # bode_plot will superimpose the data.  This was implicit
+                # before matplotlib 2.1, but changed after that (See
+                # https://github.com/matplotlib/matplotlib/issues/9024). 
+                # The code below should work on all cases.
+
+                # Get the current figure 
+                fig = plt.gcf()
+                ax_mag = None
+                ax_phase = None
+
+                # Get the current axes if they already exist
+                for ax in fig.axes:
+                    if ax.get_label() == 'control-bode-magnitude':
+                        ax_mag = ax
+                    elif ax.get_label() == 'control-bode-phase':
+                        ax_phase = ax
+
+                # If no axes present, create them from scratch
+                if ax_mag is None or ax_phase is None:
+                    plt.clf()
+                    ax_mag = plt.subplot(211, label = 'control-bode-magnitude')
+                    ax_phase = plt.subplot(212, label = 'control-bode-phase',
+                                           sharex=ax_mag)
+
                 # Magnitude plot
-                plt.subplot(211);
                 if dB:
-                    plt.semilogx(omega_plot, mag, *args, **kwargs)
+                    pltline = ax_mag.semilogx(omega_plot, 20 * np.log10(mag),
+                                              *args, **kwargs)
                 else:
-                    plt.loglog(omega_plot, mag, *args, **kwargs)
-                plt.hold(True);
+                    pltline = ax_mag.loglog(omega_plot, mag, *args, **kwargs)
+
+                if nyquistfrq_plot:
+                    ax_mag.axvline(nyquistfrq_plot,
+                                   color=pltline[0].get_color())
 
                 # Add a grid to the plot + labeling
-                plt.grid(True)
-                plt.grid(True, which='minor')
-                plt.ylabel("Magnitude (dB)" if dB else "Magnitude")
+                ax_mag.grid(True, which='both')
+                ax_mag.set_ylabel("Magnitude (dB)" if dB else "Magnitude")
 
                 # Phase plot
-                plt.subplot(212);
-                plt.semilogx(omega_plot, phase, *args, **kwargs)
-                plt.hold(True);
+                if deg:
+                    phase_plot = phase * 180. / math.pi
+                else:
+                    phase_plot = phase
+                ax_phase.semilogx(omega_plot, phase_plot, *args, **kwargs)
+
+                if nyquistfrq_plot:
+                    ax_phase.axvline(nyquistfrq_plot, color=pltline[0].get_color())
 
                 # Add a grid to the plot + labeling
-                plt.grid(True)
-                plt.grid(True, which='minor')
-                plt.ylabel("Phase (deg)" if deg else "Phase (rad)")
+                ax_phase.set_ylabel("Phase (deg)" if deg else "Phase (rad)")
+
+                def genZeroCenteredSeries(val_min, val_max, period):
+                    v1 = np.ceil(val_min / period - 0.2)
+                    v2 = np.floor(val_max / period + 0.2)
+                    return np.arange(v1, v2 + 1) * period
+                if deg:
+                    ylim = ax_phase.get_ylim()
+                    ax_phase.set_yticks(genZeroCenteredSeries(ylim[0], ylim[1], 45.))
+                    ax_phase.set_yticks(genZeroCenteredSeries(ylim[0], ylim[1], 15.), minor=True)
+                else:
+                    ylim = ax_phase.get_ylim()
+                    ax_phase.set_yticks(genZeroCenteredSeries(ylim[0], ylim[1], math.pi / 4.))
+                    ax_phase.set_yticks(genZeroCenteredSeries(ylim[0], ylim[1], math.pi / 12.), minor=True)
+                ax_phase.grid(True, which='both')
+                # ax_mag.grid(which='minor', alpha=0.3)
+                # ax_mag.grid(which='major', alpha=0.9)
+                # ax_phase.grid(which='minor', alpha=0.3)
+                # ax_phase.grid(which='major', alpha=0.9)
 
                 # Label the frequency axis
-                plt.xlabel("Frequency (Hz)" if Hz else "Frequency (rad/sec)")
+                ax_phase.set_xlabel("Frequency (Hz)" if Hz else "Frequency (rad/sec)")
 
     if len(syslist) == 1:
         return mags[0], phases[0], omegas[0]
@@ -178,13 +254,14 @@ def bode_plot(syslist, omega=None, dB=None, Hz=None, deg=None,
 # Nyquist plot
 def nyquist_plot(syslist, omega=None, Plot=True, color='b',
                  labelFreq=0, *args, **kwargs):
-    """Nyquist plot for a system
+    """
+    Nyquist plot for a system
 
     Plots a Nyquist plot for the system over a (optional) frequency range.
 
     Parameters
     ----------
-    syslist : list of Lti
+    syslist : list of LTI
         List of linear input/output systems (single system is OK)
     omega : freq_range
         Range of frequencies (list or bounds) in rad/sec
@@ -192,7 +269,7 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
         If True, plot magnitude
     labelFreq : int
         Label every nth frequency on the plot
-    *args, **kwargs:
+    \*args, \**kwargs:
         Additional options to matplotlib (color, linestyle, etc)
 
     Returns
@@ -208,6 +285,7 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
     --------
     >>> sys = ss("1. -2; 3. -4", "5.; 7", "6. 8", "9.")
     >>> real, imag, freq = nyquist_plot(sys)
+
     """
     # If argument was a singleton, turn it into a list
     if (not getattr(syslist, '__iter__', False)):
@@ -215,11 +293,10 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
 
     # Select a default range if none is provided
     if omega is None:
-        #! TODO: think about doing something smarter for discrete
         omega = default_frequency_range(syslist)
 
     # Interpolate between wmin and wmax if a tuple or list are provided
-    elif (isinstance(omega,list) | isinstance(omega,tuple)):
+    elif (isinstance(omega, list) | isinstance(omega, tuple)):
         # Only accept tuple or list of length 2
         if (len(omega) != 2):
             raise ValueError("Supported frequency arguments are (wmin,wmax) tuple or list, or frequency vector. ")
@@ -227,7 +304,7 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
                             num=50, endpoint=True, base=10.0)
     for sys in syslist:
         if (sys.inputs > 1 or sys.outputs > 1):
-            #TODO: Add MIMO nyquist plots.
+            # TODO: Add MIMO nyquist plots.
             raise NotImplementedError("Nyquist is currently only implemented for SISO systems.")
         else:
             # Get the magnitude and phase of the system
@@ -251,13 +328,13 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
                 ind = slice(None, None, labelFreq)
                 for xpt, ypt, omegapt in zip(x[ind], y[ind], omega[ind]):
                     # Convert to Hz
-                    f = omegapt/(2*sp.pi)
+                    f = omegapt / (2 * sp.pi)
 
                     # Factor out multiples of 1000 and limit the
                     # result to the range [-8, 8].
-                    pow1000 = max(min(get_pow1000(f),8),-8)
+                    pow1000 = max(min(get_pow1000(f), 8), -8)
 
-                     # Get the SI prefix.
+                    # Get the SI prefix.
                     prefix = gen_prefix(pow1000)
 
                     # Apply the text. (Use a space before the text to
@@ -267,7 +344,7 @@ def nyquist_plot(syslist, omega=None, Plot=True, color='b',
                     # instead of 1.0, and this would otherwise be
                     # truncated to 0.
                     plt.text(xpt, ypt,
-                             ' ' + str(int(np.round(f/1000**pow1000, 0))) +
+                             ' ' + str(int(np.round(f / 1000 ** pow1000, 0))) +
                              ' ' + prefix + 'Hz')
         return x, y, omega
 
@@ -281,7 +358,7 @@ def gangof4_plot(P, C, omega=None):
 
     Parameters
     ----------
-    P, C : Lti
+    P, C : LTI
         Linear input/output systems (process and control)
     omega : array
         Range of frequencies (list or bounds) in rad/sec
@@ -290,42 +367,64 @@ def gangof4_plot(P, C, omega=None):
     -------
     None
     """
-    if (P.inputs > 1 or P.outputs > 1 or C.inputs > 1 or C.outputs >1):
-        #TODO: Add MIMO go4 plots.
+    if (P.inputs > 1 or P.outputs > 1 or C.inputs > 1 or C.outputs > 1):
+        # TODO: Add MIMO go4 plots.
         raise NotImplementedError("Gang of four is currently only implemented for SISO systems.")
     else:
 
         # Select a default range if none is provided
         #! TODO: This needs to be made more intelligent
         if omega is None:
-            omega = default_frequency_range((P,C))
+            omega = default_frequency_range((P, C))
 
         # Compute the senstivity functions
-        L = P*C;
+        L = P * C;
         S = feedback(1, L);
         T = L * S;
+        
+        # Set up the axes with labels so that multiple calls to
+        # gangof4_plot will superimpose the data.  See details in bode_plot.
+        plot_axes = {'t' : None, 's' : None, 'ps' : None, 'cs' : None}
+        for ax in plt.gcf().axes:
+            label = ax.get_label()
+            if label.startswith('control-gangof4-'):
+                key = label[len('control-gangof4-'):]
+                if key not in plot_axes:
+                    raise RuntimeError("unknown gangof4 axis type '{}'".format(label))
+                plot_axes[key] = ax
 
+        # if any of the axes are missing, start from scratch
+        if any((ax is None for ax in plot_axes.values())):
+            plt.clf()
+            plot_axes = {'t' : plt.subplot(221,label='control-gangof4-t'),
+                         'ps' : plt.subplot(222,label='control-gangof4-ps'),
+                         'cs' : plt.subplot(223,label='control-gangof4-cs'),
+                         's' : plt.subplot(224,label='control-gangof4-s')}
+
+        #
         # Plot the four sensitivity functions
+        #
+
         #! TODO: Need to add in the mag = 1 lines
         mag_tmp, phase_tmp, omega = T.freqresp(omega);
         mag = np.squeeze(mag_tmp)
         phase = np.squeeze(phase_tmp)
-        plt.subplot(221); plt.loglog(omega, mag);
+        plot_axes['t'].loglog(omega, mag);
 
-        mag_tmp, phase_tmp, omega = (P*S).freqresp(omega);
+        mag_tmp, phase_tmp, omega = (P * S).freqresp(omega);
         mag = np.squeeze(mag_tmp)
         phase = np.squeeze(phase_tmp)
-        plt.subplot(222); plt.loglog(omega, mag);
+        plot_axes['ps'].loglog(omega, mag);
 
-        mag_tmp, phase_tmp, omega = (C*S).freqresp(omega);
+        mag_tmp, phase_tmp, omega = (C * S).freqresp(omega);
         mag = np.squeeze(mag_tmp)
         phase = np.squeeze(phase_tmp)
-        plt.subplot(223); plt.loglog(omega, mag);
+        plot_axes['cs'].loglog(omega, mag);
 
         mag_tmp, phase_tmp, omega = S.freqresp(omega);
         mag = np.squeeze(mag_tmp)
         phase = np.squeeze(phase_tmp)
-        plt.subplot(224); plt.loglog(omega, mag);
+        plot_axes['s'].loglog(omega, mag);
 
 #
 # Utility functions
@@ -335,7 +434,7 @@ def gangof4_plot(P, C, omega=None):
 #
 
 # Compute reasonable defaults for axes
-def default_frequency_range(syslist):
+def default_frequency_range(syslist, Hz=None, number_of_samples=None, feature_periphery_decade=None):
     """Compute a reasonable default frequency range for frequency
     domain plots.
 
@@ -344,8 +443,20 @@ def default_frequency_range(syslist):
 
     Parameters
     ----------
-    syslist : list of Lti
+    syslist : list of LTI
         List of linear input/output systems (single system is OK)
+    Hz: boolean
+        If True, the limits (first and last value) of the frequencies
+        are set to full decades in Hz so it fits plotting with logarithmic
+        scale in Hz otherwise in rad/s. Omega is always returned in rad/sec.
+    number_of_samples: int
+        Number of samples to generate
+    feature_periphery_decade: float
+        Defines how many decades shall be included in the frequency range on
+        both sides of features (poles, zeros).
+        Example: If there is a feature, e.g. a pole, at 1Hz and feature_periphery_decade=1.
+        then the range of frequencies shall span 0.1 .. 10 Hz.
+        The default value is read from config.bode_feature_periphery_decade.
 
     Returns
     -------
@@ -364,8 +475,16 @@ def default_frequency_range(syslist):
     # integer.  It excludes poles and zeros at the origin.  If no features
     # are found, it turns logspace(-1, 1)
 
+    # Set default values for options
+    from . import config
+    if (number_of_samples is None):
+        number_of_samples = config.bode_number_of_samples
+    if (feature_periphery_decade is None):
+        feature_periphery_decade = config.bode_feature_periphery_decade
+
     # Find the list of all poles and zeros in the systems
     features = np.array(())
+    freq_interesting = []
 
     # detect if single sys passed by checking if it is sequence-like
     if (not getattr(syslist, '__iter__', False)):
@@ -374,26 +493,60 @@ def default_frequency_range(syslist):
     for sys in syslist:
         try:
             # Add new features to the list
-            features = np.concatenate((features, np.abs(sys.pole())))
-            features = np.concatenate((features, np.abs(sys.zero())))
+            if sys.isctime():
+                features_ = np.concatenate((np.abs(sys.pole()),
+                                           np.abs(sys.zero())))
+                # Get rid of poles and zeros at the origin
+                features_ = features_[features_ != 0.0];
+                features = np.concatenate((features, features_))
+            elif sys.isdtime(strict=True):
+                fn = math.pi * 1. / sys.dt
+                # TODO: What distance to the Nyquist frequency is appropriate?
+                freq_interesting.append(fn * 0.9)
+
+                features_ = np.concatenate((sys.pole(),
+                                           sys.zero()))
+                # Get rid of poles and zeros
+                # * at the origin and real <= 0 & imag==0: log!
+                # * at 1.: would result in omega=0. (logaritmic plot!)
+                features_ = features_[(features_.imag != 0.0) | (features_.real > 0.)]
+                features_ = features_[np.bitwise_not((features_.imag == 0.0) & (np.abs(features_.real - 1.0) < 1.e-10))]
+                # TODO: improve
+                features__ = np.abs(np.log(features_) / (1.j * sys.dt))
+                features = np.concatenate((features, features__))
+            else:
+                # TODO
+                raise NotImplementedError('type of system in not implemented now')
         except:
             pass
 
-    # Get rid of poles and zeros at the origin
-    features = features[features != 0];
 
     # Make sure there is at least one point in the range
-    if (features.shape[0] == 0): features = [1];
+    if (features.shape[0] == 0):
+        features = np.array([1.]);
 
-    # Take the log of the features
-    features = np.log10(features)
+    if Hz:
+        features /= 2.*math.pi
+        features = np.log10(features)
+        lsp_min = np.floor(np.min(features) - feature_periphery_decade)
+        lsp_max = np.ceil(np.max(features) + feature_periphery_decade)
+        lsp_min += np.log10(2.*math.pi)
+        lsp_max += np.log10(2.*math.pi)
+    else:
+        features = np.log10(features)
+        lsp_min = np.floor(np.min(features) - feature_periphery_decade)
+        lsp_max = np.ceil(np.max(features) + feature_periphery_decade)
+    if freq_interesting:
+        lsp_min = min(lsp_min, np.log10(min(freq_interesting)))
+        lsp_max = max(lsp_max, np.log10(max(freq_interesting)))
 
-    #! TODO: Add a check in discrete case to make sure we don't get aliasing
+    #! TODO: Add a check in discrete case to make sure we don't get aliasing (Attention: there is a list of system but only one omega vector)
 
     # Set the range to be an order of magnitude beyond any features
-    omega = sp.logspace(np.floor(np.min(features))-1,
-                        np.ceil(np.max(features))+1)
-
+    if number_of_samples:
+        omega = sp.logspace(lsp_min, lsp_max, num=number_of_samples, endpoint=True)
+    else:
+        omega = sp.logspace(lsp_min, lsp_max, endpoint=True)
     return omega
 
 #
@@ -412,7 +565,7 @@ def get_pow1000(num):
         return 0
     elif dnum < 0:
         dnum = -dnum
-    return int(floor(dnum.log10()/3))
+    return int(floor(dnum.log10() / 3))
 
 def gen_prefix(pow1000):
     '''Return the SI prefix for a power of 1000.
@@ -421,23 +574,23 @@ def gen_prefix(pow1000):
     # deca, deci, and centi).
     if pow1000 < -8 or pow1000 > 8:
         raise ValueError("Value is out of the range covered by the SI prefixes.")
-    return ['Y', # yotta (10^24)
-            'Z', # zetta (10^21)
-            'E', # exa (10^18)
-            'P', # peta (10^15)
-            'T', # tera (10^12)
-            'G', # giga (10^9)
-            'M', # mega (10^6)
-            'k', # kilo (10^3)
-            '', # (10^0)
-            'm', # milli (10^-3)
-            r'$\mu$', # micro (10^-6)
-            'n', # nano (10^-9)
-            'p', # pico (10^-12)
-            'f', # femto (10^-15)
-            'a', # atto (10^-18)
-            'z', # zepto (10^-21)
-            'y'][8 - pow1000] # yocto (10^-24)
+    return ['Y',  # yotta (10^24)
+            'Z',  # zetta (10^21)
+            'E',  # exa (10^18)
+            'P',  # peta (10^15)
+            'T',  # tera (10^12)
+            'G',  # giga (10^9)
+            'M',  # mega (10^6)
+            'k',  # kilo (10^3)
+            '',  # (10^0)
+            'm',  # milli (10^-3)
+            r'$\mu$',  # micro (10^-6)
+            'n',  # nano (10^-9)
+            'p',  # pico (10^-12)
+            'f',  # femto (10^-15)
+            'a',  # atto (10^-18)
+            'z',  # zepto (10^-21)
+            'y'][8 - pow1000]  # yocto (10^-24)
 
 # Function aliases
 bode = bode_plot
