@@ -7,9 +7,10 @@ import unittest
 import numpy as np
 from control.statesp import StateSpace, _convertToStateSpace
 from control.xferfcn import TransferFunction, _convertToTransferFunction
+from control.lti import evalfr
+from control.exception import slycot_check
 # from control.lti import isdtime
 
-
 class TestXferFcn(unittest.TestCase):
     """These are tests for functionality and correct reporting of the transfer
     function class.  Throughout these tests, we will give different input
@@ -95,7 +96,6 @@ class TestXferFcn(unittest.TestCase):
 
     # Tests for TransferFunction.__neg__
 
-    @unittest.skip("skipping, known issue with Python 3")
     def testNegScalar(self):
         """Negate a direct feedthrough system."""
 
@@ -114,6 +114,8 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys2.num, [[[-1., -3., -5.]]])
         np.testing.assert_array_equal(sys2.den, [[[1., 6., 2., -1.]]])
 
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testNegMIMO(self):
         """Negate a MIMO system."""
 
@@ -156,6 +158,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys3.num, [[[20., 4., -8]]])
         np.testing.assert_array_equal(sys3.den, [[[1., 6., 1., -7., -2., 1.]]])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testAddMIMO(self):
         """Add two MIMO systems."""
 
@@ -206,6 +209,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys4.num, [[[-2., -6., 12., 10., 2.]]])
         np.testing.assert_array_equal(sys4.den, [[[1., 6., 1., -7., -2., 1.]]])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testSubMIMO(self):
         """Add two MIMO systems."""
 
@@ -259,6 +263,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys3.num, sys4.num)
         np.testing.assert_array_equal(sys3.den, sys4.den)
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMulMIMO(self):
         """Multiply two MIMO systems."""
 
@@ -315,15 +320,14 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys4.den, sys3.num)
 
     # Tests for TransferFunction.evalfr.
-
     def testEvalFrSISO(self):
         """Evaluate the frequency response of a SISO system at one frequency."""
 
         sys = TransferFunction([1., 3., 5], [1., 6., 2., -1])
 
-        np.testing.assert_array_almost_equal(sys.evalfr(1.),
+        np.testing.assert_array_almost_equal(evalfr(sys, 1j),
             np.array([[-0.5 - 0.5j]]))
-        np.testing.assert_array_almost_equal(sys.evalfr(32.),
+        np.testing.assert_array_almost_equal(evalfr(sys, 32j),
             np.array([[0.00281959302585077 - 0.030628473607392j]]))
 
         # Test call version as well
@@ -331,6 +335,21 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_almost_equal(sys(32.j),
             0.00281959302585077 - 0.030628473607392j)
 
+        # Test internal version (with real argument)
+        np.testing.assert_array_almost_equal(sys._evalfr(1.),
+            np.array([[-0.5 - 0.5j]]))
+        np.testing.assert_array_almost_equal(sys._evalfr(32.),
+            np.array([[0.00281959302585077 - 0.030628473607392j]]))
+
+        # Deprecated version of the call (should generate warning)
+        import warnings
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            sys.evalfr(1.)
+            assert len(w) == 1
+            assert issubclass(w[-1].category, PendingDeprecationWarning)
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testEvalFrMIMO(self):
         """Evaluate the frequency response of a MIMO system at one frequency."""
 
@@ -343,7 +362,7 @@ class TestXferFcn(unittest.TestCase):
                 [-0.083333333333333, -0.188235294117647 - 0.847058823529412j,
                  -1. - 8.j]]
 
-        np.testing.assert_array_almost_equal(sys.evalfr(2.), resp)
+        np.testing.assert_array_almost_equal(sys._evalfr(2.), resp)
 
         # Test call version as well
         np.testing.assert_array_almost_equal(sys(2.j), resp)
@@ -367,7 +386,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_almost_equal(phase, truephase)
         np.testing.assert_array_almost_equal(omega, trueomega)
 
-    @unittest.skip("skipping, known issue with Python 3")
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testFreqRespMIMO(self):
         """Evaluate the magnitude and phase of a MIMO system at multiple
         frequencies."""
@@ -397,7 +416,8 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(omega, trueomega)
 
     # Tests for TransferFunction.pole and TransferFunction.zero.
-
+    
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testPoleMIMO(self):
         """Test for correct MIMO poles."""
 
@@ -405,10 +425,16 @@ class TestXferFcn(unittest.TestCase):
             [[[1., 2.], [1., 3.]], [[1., 4., 4.], [1., 9., 14.]]])
         p = sys.pole()
 
-        np.testing.assert_array_almost_equal(p, [-7., -3., -2., -2.])
-
-    # Tests for TransferFunction.feedback.
+        np.testing.assert_array_almost_equal(p, [-2., -2., -7., -3., -2.])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
+    def testDoubleCancelingPoleSiso(self):
+        
+        H = TransferFunction([1,1],[1,2,1])
+        p = H.pole()
+        np.testing.assert_array_almost_equal(p, [-1, -1])
+    
+    # Tests for TransferFunction.feedback
     def testFeedbackSISO(self):
         """Test for correct SISO transfer function feedback."""
 
@@ -423,6 +449,7 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_equal(sys4.num, [[[-1., 7., -16., 16., 0.]]])
         np.testing.assert_array_equal(sys4.den, [[[1., 0., 2., -8., 8., 0.]]])
 
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testConvertToTransferFunction(self):
         """Test for correct state space to transfer function conversion."""
 
@@ -454,6 +481,7 @@ class TestXferFcn(unittest.TestCase):
         hr = (s+1)/(s**2+s+1)
         np.testing.assert_array_almost_equal(hm.num[0][0], hr.num[0][0])
         np.testing.assert_array_almost_equal(hm.den[0][0], hr.den[0][0])
+        np.testing.assert_equal(hm.dt, hr.dt)
 
     def testMinreal2(self):
         """This one gave a problem, due to poly([]) giving simply 1
@@ -467,7 +495,26 @@ class TestXferFcn(unittest.TestCase):
         hr = 6205/(s**2+8*s+1241)
         np.testing.assert_array_almost_equal(H2b.num[0][0], hr.num[0][0])
         np.testing.assert_array_almost_equal(H2b.den[0][0], hr.den[0][0])
+        np.testing.assert_equal(H2b.dt, hr.dt)
 
+    def testMinreal3(self):
+        """Regression test for minreal of tf([1,1],[1,1])"""
+        g = TransferFunction([1,1],[1,1]).minreal()
+        np.testing.assert_array_almost_equal(1.0, g.num[0][0])
+        np.testing.assert_array_almost_equal(1.0, g.den[0][0])
+        np.testing.assert_equal(None, g.dt)
+
+    def testMinreal4(self):
+        """Check minreal on discrete TFs."""
+        T = 0.01
+        z = TransferFunction([1, 0], [1], T)
+        h = (z-1.00000000001)*(z+1.0000000001)/((z**2-1))
+        hm = h.minreal()
+        hr = TransferFunction([1], [1], T)
+        np.testing.assert_array_almost_equal(hm.num[0][0], hr.num[0][0])
+        np.testing.assert_equal(hr.dt, hm.dt)
+
+    @unittest.skipIf(not slycot_check(), "slycot not installed")
     def testMIMO(self):
         """Test conversion of a single input, two-output state-space
         system against the same TF"""
@@ -510,6 +557,43 @@ class TestXferFcn(unittest.TestCase):
         np.testing.assert_array_almost_equal(H.num[1][0], H2.num[0][0])
         np.testing.assert_array_almost_equal(H.den[1][0], H2.den[0][0])
 
+    def test_dcgain_cont(self):
+        """Test DC gain for continuous-time transfer functions"""
+        sys = TransferFunction(6, 3)
+        np.testing.assert_equal(sys.dcgain(), 2)
+
+        sys2 = TransferFunction(6, [1, 3])
+        np.testing.assert_equal(sys2.dcgain(), 2)
+
+        sys3 = TransferFunction(6, [1, 0])
+        np.testing.assert_equal(sys3.dcgain(), np.inf)
+
+        num = [[[15], [21], [33]], [[10], [14], [22]]]
+        den = [[[1, 3], [2, 3], [3, 3]], [[1, 5], [2, 7], [3, 11]]]
+        sys4 = TransferFunction(num, den)
+        expected = [[5, 7, 11], [2, 2, 2]]
+        np.testing.assert_array_equal(sys4.dcgain(), expected)
+
+    def test_dcgain_discr(self):
+        """Test DC gain for discrete-time transfer functions"""
+        # static gain
+        sys = TransferFunction(6, 3, True)
+        np.testing.assert_equal(sys.dcgain(), 2)
+
+        # averaging filter
+        sys = TransferFunction(0.5, [1, -0.5], True)
+        np.testing.assert_almost_equal(sys.dcgain(), 1)
+
+        # differencer
+        sys = TransferFunction(1, [1, -1], True)
+        np.testing.assert_equal(sys.dcgain(), np.inf)
+
+        # summer
+        # causes a RuntimeWarning due to the divide by zero
+        sys = TransferFunction([1,-1], [1], True)
+        np.testing.assert_equal(sys.dcgain(), 0)
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(TestXferFcn)
 
