@@ -1,78 +1,12 @@
 # timeresp.py - time-domain simulation routes
-"""
-Time domain simulation.
+"""Time domain simulation.
 
-This file contains a collection of functions that calculate
-time responses for linear systems.
+This file contains a collection of functions that calculate time
+responses for linear systems.
 
-.. _time-series-convention:
+See doc/conventions.rst#time-series-conventions_ for more information
+on how time series data are represented.
 
-Convention for Time Series
---------------------------
-
-This is a convention for function arguments and return values that
-represent time series: sequences of values that change over time. It
-is used throughout the library, for example in the functions
-:func:`forced_response`, :func:`step_response`, :func:`impulse_response`,
-and :func:`initial_response`.
-
-.. note::
-    This convention is different from the convention used in the library
-    :mod:`scipy.signal`. In Scipy's convention the meaning of rows and columns
-    is interchanged.  Thus, all 2D values must be transposed when they are
-    used with functions from :mod:`scipy.signal`.
-
-Types:
-
-    * **Arguments** can be **arrays**, **matrices**, or **nested lists**.
-    * **Return values** are **arrays** (not matrices).
-
-The time vector is either 1D, or 2D with shape (1, n)::
-
-      T = [[t1,     t2,     t3,     ..., tn    ]]
-
-Input, state, and output all follow the same convention. Columns are different
-points in time, rows are different components. When there is only one row, a
-1D object is accepted or returned, which adds convenience for SISO systems::
-
-      U = [[u1(t1), u1(t2), u1(t3), ..., u1(tn)]
-           [u2(t1), u2(t2), u2(t3), ..., u2(tn)]
-           ...
-           ...
-           [ui(t1), ui(t2), ui(t3), ..., ui(tn)]]
-
-      Same for X, Y
-
-So, U[:,2] is the system's input at the third point in time; and U[1] or U[1,:]
-is the sequence of values for the system's second input.
-
-The initial conditions are either 1D, or 2D with shape (j, 1)::
-
-     X0 = [[x1]
-           [x2]
-           ...
-           ...
-           [xj]]
-
-As all simulation functions return *arrays*, plotting is convenient::
-
-    t, y = step(sys)
-    plot(t, y)
-
-The output of a MIMO system can be plotted like this::
-
-    t, y, x = lsim(sys, u, t)
-    plot(t, y[0], label='y_0')
-    plot(t, y[1], label='y_1')
-
-The convention also works well with the state space form of linear systems. If
-``D`` is the feedthrough *matrix* of a linear system, and ``U`` is its input
-(*matrix* or *array*), then the feedthrough part of the system's response,
-can be computed like this::
-
-    ft = D * U
-
-----------------------------------------------------------------
 """
 
 """Copyright (c) 2011 by California Institute of Technology
@@ -120,10 +54,12 @@ import scipy as sp              # SciPy library (used all over)
 import numpy as np              # NumPy library
 from scipy.signal.ltisys import _default_response_times
 import warnings
-from .lti import Lti     # base class of StateSpace, TransferFunction
+from .lti import LTI     # base class of StateSpace, TransferFunction
 from .statesp import _convertToStateSpace, _mimo2simo, _mimo2siso
 from .lti import isdtime, isctime
 
+__all__ = ['forced_response', 'step_response', 'initial_response',
+           'impulse_response']
 
 # Helper function for checking array-like parameters
 def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
@@ -233,7 +169,7 @@ def _check_convert_array(in_obj, legal_shapes, err_msg_start, squeeze=False,
 
 
 # Forced response of a linear system
-def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
+def forced_response(sys, T=None, U=0., X0=0., transpose=False):
     """Simulate the output of a linear system.
 
     As a convenience for parameters `U`, `X0`:
@@ -241,11 +177,11 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
     The correct shape is inferred from arguments `sys` and `T`.
 
     For information on the **shape** of parameters `U`, `T`, `X0` and
-    return values `T`, `yout`, `xout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, `xout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
-    sys: Lti (StateSpace, or TransferFunction)
+    sys: LTI (StateSpace, or TransferFunction)
         LTI system to simulate
 
     T: array-like
@@ -264,13 +200,6 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
-
     Returns
     -------
     T: array
@@ -287,9 +216,11 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
     Examples
     --------
     >>> T, yout, xout = forced_response(sys, T, u, X0)
+
+    See :ref:`time-series-convention`.
     """
-    if not isinstance(sys, Lti):
-        raise TypeError('Parameter ``sys``: must be a ``Lti`` object. '
+    if not isinstance(sys, LTI):
+        raise TypeError('Parameter ``sys``: must be a ``LTI`` object. '
                         '(For example ``StateSpace`` or ``TransferFunction``)')
     sys = _convertToStateSpace(sys)
     A, B, C, D = np.asarray(sys.A), np.asarray(sys.B), np.asarray(sys.C), \
@@ -393,6 +324,10 @@ def forced_response(sys, T=None, U=0., X0=0., transpose=False, **keywords):
         dsys = (A, B, C, D, sys.dt)
         tout, yout, xout = sp.signal.dlsim(dsys, U, T, X0)
 
+        # Transpose the output and state vectors to match local convention
+        xout = sp.transpose(xout)
+        yout = sp.transpose(yout)
+
     # See if we need to transpose the data back into MATLAB form
     if (transpose):
         T = np.transpose(T)
@@ -420,7 +355,7 @@ def _get_ss_simo(sys, input=None, output=None):
         return _mimo2siso(sys_ss, input, output, warn_conversion=warn)
 
 def step_response(sys, T=None, X0=0., input=None, output=None,
-                  transpose=False, **keywords):
+                  transpose=False, return_x=False):
     # pylint: disable=W0622
     """Step response of a linear system
 
@@ -430,7 +365,7 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     inputs are set to 0, all other outputs are ignored.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -456,13 +391,8 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -472,6 +402,9 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
     yout: array
         Response of the system
 
+    xout: array
+        Individual response of each x variable
+
     See Also
     --------
     forced_response, initial_response, impulse_response
@@ -491,14 +424,17 @@ def step_response(sys, T=None, X0=0., input=None, output=None,
 
     U = np.ones_like(T)
 
-    T, yout, _xout = forced_response(sys, T, U, X0,
-                                     transpose=transpose, **keywords)
+    T, yout, xout = forced_response(sys, T, U, X0,
+                                     transpose=transpose)
+
+    if return_x:
+        return T, yout, xout
 
     return T, yout
 
 
 def initial_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, **keywords):
+                     transpose=False, return_x=False):
     # pylint: disable=W0622
     """Initial condition response of a linear system
 
@@ -507,7 +443,7 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
     outputs are given.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -534,14 +470,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
-
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -549,6 +479,8 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
         Time values of the output
     yout: array
         Response of the system
+    xout: array
+        Individual response of each x variable
 
     See Also
     --------
@@ -566,13 +498,16 @@ def initial_response(sys, T=None, X0=0., input=0, output=None,
         T = _default_response_times(sys.A, 100)
     U = np.zeros_like(T)
 
-    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose,
-                                     **keywords)
+    T, yout, _xout = forced_response(sys, T, U, X0, transpose=transpose)
+
+    if return_x:
+        return T, yout, _xout
+
     return T, yout
 
 
 def impulse_response(sys, T=None, X0=0., input=0, output=None,
-                     transpose=False, **keywords):
+                     transpose=False, return_x=False):
     # pylint: disable=W0622
     """Impulse response of a linear system
 
@@ -582,7 +517,7 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     inputs are set to 0, all other outputs are ignored.
 
     For information on the **shape** of parameters `T`, `X0` and
-    return values `T`, `yout` see: :ref:`time-series-convention`
+    return values `T`, `yout`, see :ref:`time-series-convention`.
 
     Parameters
     ----------
@@ -608,14 +543,8 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
         If True, transpose all input and output arrays (for backward
         compatibility with MATLAB and scipy.signal.lsim)
 
-    **keywords:
-        Additional keyword arguments control the solution algorithm for the
-        differential equations. These arguments are passed on to the function
-        :func:`lsim`, which in turn passes them on to
-        :func:`scipy.integrate.odeint`. See the documentation for
-        :func:`scipy.integrate.odeint` for information about these
-        arguments.
-
+    return_x: bool
+        If True, return the state vector (default = False).
 
     Returns
     -------
@@ -623,10 +552,12 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
         Time values of the output
     yout: array
         Response of the system
+    xout: array
+        Individual response of each x variable
 
     See Also
     --------
-    ForcedReponse, initial_response, step_response
+    forced_response, initial_response, step_response
 
     Examples
     --------
@@ -646,19 +577,28 @@ def impulse_response(sys, T=None, X0=0., input=0, output=None,
     X0 = _check_convert_array(X0, [(n_states,), (n_states, 1)],
                               'Parameter ``X0``: \n', squeeze=True)
 
-    # Compute new X0 that contains the impulse
-    # We can't put the impulse into U because there is no numerical
-    # representation for it (infinitesimally short, infinitely high).
-    # See also: http://www.mathworks.com/support/tech-notes/1900/1901.html
-    B = np.asarray(sys.B).squeeze()
-    new_X0 = B + X0
-
     # Compute T and U, no checks necessary, they will be checked in lsim
     if T is None:
         T = _default_response_times(sys.A, 100)
     U = np.zeros_like(T)
 
+    # Compute new X0 that contains the impulse
+    # We can't put the impulse into U because there is no numerical
+    # representation for it (infinitesimally short, infinitely high).
+    # See also: http://www.mathworks.com/support/tech-notes/1900/1901.html
+    if isctime(sys):
+        B = np.asarray(sys.B).squeeze()
+        new_X0 = B + X0
+    else:
+        new_X0 = X0
+        U[0] = 1.
+        print("discrete", U)
+
     T, yout, _xout = forced_response(
         sys, T, U, new_X0,
-        transpose=transpose, **keywords)
+        transpose=transpose)
+
+    if return_x:
+        return T, yout, _xout
+
     return T, yout
